from django.contrib.postgres.fields import ArrayField
from django.db import models
from django.urls import reverse

from netbox.models import NetBoxModel
from utilities.choices import ChoiceSet
from .utils import file_upload

class SiteDocTypeChoices(ChoiceSet):

    key = 'DocTypeChoices.site'

    CHOICES = [
        ('diagram', 'Network Diagram', 'green'),
        ('floorplan', 'Floor Plan', 'purple'),
        ('purchaseorder', 'Purchase Order', 'orange'),
        ('quote', 'Quote', 'indigo'),
        ('wirelessmodel', 'Wireless Model (Ekahau)', 'yellow'),
        ('other', 'Other', 'gray'),
    ]

class DeviceDocTypeChoices(ChoiceSet):

    key = 'DocTypeChoices.device'

    CHOICES = [
        ('diagram', 'Network Diagram', 'green'),
        ('manual', 'Manual', 'pink'),
        ('purchaseorder', 'Purchase Order', 'orange'),
        ('quote', 'Quote', 'indigo'),
        ('supportcontract', 'Support Contract', 'blue'),
        ('other', 'Other', 'gray'),
    ]


class DeviceTypeDocTypeChoices(ChoiceSet):

    key = 'DocTypeChoices.devicetype'

    CHOICES = [
        ('diagram', 'Network Diagram', 'green'),
        ('manual', 'Manual', 'pink'),
        ('purchaseorder', 'Purchase Order', 'orange'),
        ('quote', 'Quote', 'indigo'),
        ('supportcontract', 'Support Contract', 'blue'),
        ('other', 'Other', 'gray'),
    ]

class CircuitDocTypeChoices(ChoiceSet):
    
    key = 'DocTypeChoices.circuit'

    CHOICES = [
        ('circuitcontract', 'Circuit Contract', 'red'),
        ('diagram', 'Network Diagram', 'green'),
        ('purchaseorder', 'Purchase Order', 'orange'),
        ('quote', 'Quote', 'indigo'),
        ('other', 'Other', 'gray'),
    ]

class SiteDocument(NetBoxModel):
    name = models.CharField(
        max_length=100,
        blank=True,
        help_text='(Optional) Specify a name to display for this document. If no name is specified, the filename will be used.'
    )
    document = models.FileField(
        upload_to=file_upload
    )
    document_type = models.CharField(
        max_length=30,
        choices=SiteDocTypeChoices
    )

    site = models.ForeignKey(
        to='dcim.Site',
        on_delete=models.CASCADE,
        related_name='documents'
    )

    comments = models.TextField(
        blank=True
    )

    class Meta:
        ordering = ('-created', 'name')

    def get_document_type_color(self):
        return SiteDocTypeChoices.colors.get(self.document_type)

    @property
    def size(self):
        """
        Wrapper around `document.size` to suppress an OSError in case the file is inaccessible. Also opportunistically
        catch other exceptions that we know other storage back-ends to throw.
        """
        expected_exceptions = [OSError]

        try:
            from botocore.exceptions import ClientError
            expected_exceptions.append(ClientError)
        except ImportError:
            pass

        try:
            return self.document.size
        except tuple(expected_exceptions):
            return None

    @property
    def filename(self):
        filename = self.document.name.rsplit('/', 1)[-1]
        return filename.split('_', 1)[1]

    def __str__(self):
        if self.name:
            return self.name
        filename = self.document.name.rsplit('/', 1)[-1]
        return filename.split('_', 1)[1]

    def get_absolute_url(self):
        return reverse('plugins:netbox_documents:sitedocument', args=[self.pk])

    def delete(self, *args, **kwargs):

        _name = self.document.name

        # Delete file from disk
        super().delete(*args, **kwargs)
        self.document.delete(save=False)

        # Restore the name of the document as it's re-used in the notifications later
        self.document.name = _name


class DeviceDocument(NetBoxModel):
    name = models.CharField(
        max_length=100,
        blank=True,
        help_text='(Optional) Specify a name to display for this document. If no name is specified, the filename will be used.'
    )

    document = models.FileField(
        upload_to=file_upload
    )
    document_type = models.CharField(
        max_length=30,
        choices=DeviceDocTypeChoices
    )

    device = models.ForeignKey(
        to='dcim.Device',
        on_delete=models.CASCADE,
        related_name='documents'
    )

    comments = models.TextField(
        blank=True
    )

    class Meta:
        ordering = ('name',)

    def get_document_type_color(self):
        return DeviceDocTypeChoices.colors.get(self.document_type)

    @property
    def size(self):
        """
        Wrapper around `document.size` to suppress an OSError in case the file is inaccessible. Also opportunistically
        catch other exceptions that we know other storage back-ends to throw.
        """
        expected_exceptions = [OSError]

        try:
            from botocore.exceptions import ClientError
            expected_exceptions.append(ClientError)
        except ImportError:
            pass

        try:
            return self.document.size
        except tuple(expected_exceptions):
            return None

    @property
    def filename(self):
        filename = self.document.name.rsplit('/', 1)[-1]
        return filename.split('_', 1)[1]

    def __str__(self):
        if self.name:
            return self.name
        filename = self.document.name.rsplit('/', 1)[-1]
        return filename.split('_', 1)[1]


    def get_absolute_url(self):
        return reverse('plugins:netbox_documents:devicedocument', args=[self.pk])

    def delete(self, *args, **kwargs):

        _name = self.document.name

        # Delete file from disk
        super().delete(*args, **kwargs)
        self.document.delete(save=False)

        # Restore the name of the document as it's re-used in the notifications later
        self.document.name = _name


class DeviceTypeDocument(NetBoxModel):
    name = models.CharField(
        max_length=100,
        blank=True,
        help_text='(Optional) Specify a name to display for this document. If no name is specified, the filename will be used.'
    )

    document = models.FileField(
        upload_to=file_upload
    )
    
    document_type = models.CharField(
        max_length=30,
        choices=DeviceTypeDocTypeChoices
    )

    device_type = models.ForeignKey(
        to='dcim.DeviceType',
        on_delete=models.CASCADE,
        related_name='documents'
    )

    comments = models.TextField(
        blank=True
    )

    class Meta:
        ordering = ('name',)

    def get_document_type_color(self):
        return DeviceTypeDocTypeChoices.colors.get(self.document_type)

    @property
    def size(self):
        """
        Wrapper around `document.size` to suppress an OSError in case the file is inaccessible. Also opportunistically
        catch other exceptions that we know other storage back-ends to throw.
        """
        expected_exceptions = [OSError]

        try:
            from botocore.exceptions import ClientError
            expected_exceptions.append(ClientError)
        except ImportError:
            pass

        try:
            return self.document.size
        except tuple(expected_exceptions):
            return None

    @property
    def filename(self):
        filename = self.document.name.rsplit('/', 1)[-1]
        return filename.split('_', 1)[1]

    def __str__(self):
        if self.name:
            return self.name
        filename = self.document.name.rsplit('/', 1)[-1]
        return filename.split('_', 1)[1]


    def get_absolute_url(self):
        return reverse('plugins:netbox_documents:devicetypedocument', args=[self.pk])

    def delete(self, *args, **kwargs):

        _name = self.document.name

        # Delete file from disk
        super().delete(*args, **kwargs)
        self.document.delete(save=False)

        # Restore the name of the document as it's re-used in the notifications later
        self.document.name = _name

class CircuitDocument(NetBoxModel):
    name = models.CharField(
        max_length=100,
        blank=True,
        help_text='(Optional) Specify a name to display for this document. If no name is specified, the filename will be used.'
    )
    document = models.FileField(
        upload_to=file_upload
    )
    document_type = models.CharField(
        max_length=30,
        choices=CircuitDocTypeChoices
    )

    circuit = models.ForeignKey(
        to='circuits.Circuit',
        on_delete=models.CASCADE,
        related_name='documents'
    )

    comments = models.TextField(
        blank=True
    )

    def get_document_type_color(self):
        return CircuitDocTypeChoices.colors.get(self.document_type)

    class Meta:
        ordering = ('name',)

    @property
    def size(self):
        """
        Wrapper around `document.size` to suppress an OSError in case the file is inaccessible. Also opportunistically
        catch other exceptions that we know other storage back-ends to throw.
        """
        expected_exceptions = [OSError]

        try:
            from botocore.exceptions import ClientError
            expected_exceptions.append(ClientError)
        except ImportError:
            pass

        try:
            return self.document.size
        except tuple(expected_exceptions):
            return None

    @property
    def filename(self):
        filename = self.document.name.rsplit('/', 1)[-1]
        return filename.split('_', 1)[1]

    def __str__(self):
        if self.name:
            return self.name
        filename = self.document.name.rsplit('/', 1)[-1]
        return filename.split('_', 1)[1]

    def get_absolute_url(self):
        return reverse('plugins:netbox_documents:circuitdocument', args=[self.pk])

    def delete(self, *args, **kwargs):

        _name = self.document.name

        # Delete file from disk
        super().delete(*args, **kwargs)
        self.document.delete(save=False)

        # Restore the name of the document as it's re-used in the notifications later
        self.document.name = _name



from django.urls import path
from . import models, views
from netbox.views.generic import ObjectChangeLogView

urlpatterns = (

    # SiteDocument
    path('site-document/', views.SiteDocumentListView.as_view(), name='sitedocument_list'),
    path('site-document/add/', views.SiteDocumentEditView.as_view(), name='sitedocument_add'),
    path('site-document/<int:pk>/', views.SiteDocumentView.as_view(), name='sitedocument'),
    path('site-document/<int:pk>/edit/', views.SiteDocumentEditView.as_view(), name='sitedocument_edit'),
    path('site-document/<int:pk>/delete/', views.SiteDocumentDeleteView.as_view(), name='sitedocument_delete'),
    path('site-document/<int:pk>/changelog/', ObjectChangeLogView.as_view(), name='sitedocument_changelog', kwargs={
        'model': models.SiteDocument
    }),

    # DeviceDocument
    path('device-document/', views.DeviceDocumentListView.as_view(), name='devicedocument_list'),
    path('device-document/add/', views.DeviceDocumentEditView.as_view(), name='devicedocument_add'),
    path('device-document/<int:pk>/', views.DeviceDocumentView.as_view(), name='devicedocument'),
    path('device-document/<int:pk>/edit/', views.DeviceDocumentEditView.as_view(), name='devicedocument_edit'),
    path('device-document/<int:pk>/delete/', views.DeviceDocumentDeleteView.as_view(), name='devicedocument_delete'),
    path('device-document/<int:pk>/changelog/', ObjectChangeLogView.as_view(), name='devicedocument_changelog', kwargs={
        'model': models.DeviceDocument
    }),

    
    # DeviceTypeDocument
    path('device-type-document/', views.DeviceTypeDocumentListView.as_view(), name='devicetypedocument_list'),
    path('device-type-document/add/', views.DeviceTypeDocumentEditView.as_view(), name='devicetypedocument_add'),
    path('device-type-document/<int:pk>/', views.DeviceTypeDocumentView.as_view(), name='devicetypedocument'),
    path('device-type-document/<int:pk>/edit/', views.DeviceTypeDocumentEditView.as_view(), name='devicetypedocument_edit'),
    path('device-type-document/<int:pk>/delete/', views.DeviceTypeDocumentDeleteView.as_view(), name='devicetypedocument_delete'),
    path('device-type-document/<int:pk>/changelog/', ObjectChangeLogView.as_view(), name='devicetypedocument_changelog', kwargs={
        'model': models.DeviceTypeDocument
    }),

    # CircuitDocument
    path('circuit-document/', views.CircuitDocumentListView.as_view(), name='circuitdocument_list'),
    path('circuit-document/add/', views.CircuitDocumentEditView.as_view(), name='circuitdocument_add'),
    path('circuit-document/<int:pk>/', views.CircuitDocumentView.as_view(), name='circuitdocument'),
    path('circuit-document/<int:pk>/edit/', views.CircuitDocumentEditView.as_view(), name='circuitdocument_edit'),
    path('circuit-document/<int:pk>/delete/', views.CircuitDocumentDeleteView.as_view(), name='circuitdocument_delete'),
    path('circuit-document/<int:pk>/changelog/', ObjectChangeLogView.as_view(), name='circuitdocument_changelog', kwargs={
        'model': models.CircuitDocument
    }), 
    

)




def file_upload(instance, filename):

    """
    Return a path for uploading image attchments.
    Adapted from netbox/extras/utils.py
    """
    path = 'netbox-documents/'

    if hasattr(instance, 'site'):
        path_prepend = instance.site.id
    if hasattr(instance, 'device'):
        path_prepend = instance.device.id
    if hasattr(instance, 'device_type'): 
        path_prepend = instance.device_type.id
    if hasattr(instance, 'circuit'):
        path_prepend = instance.circuit.id

    # Rename the file to the provided name, if any. Attempt to preserve the file extension.
    extension = filename.rsplit('.')[-1].lower()
    if instance.name and extension in ['bmp', 'gif', 'jpeg', 'jpg', 'png', 'pdf', 'txt', 'doc', 'docx', 'xls', 'xlsx', 'xlsm']:
        filename = '.'.join([instance.name, extension])
    elif instance.name:
        filename = instance.name

    return '{}{}_{}'.format(path, path_prepend, filename)



from netbox.api.routers import NetBoxRouter
from . import views

app_name = 'netbox_documents'

router = NetBoxRouter()
router.register('site-documents', views.SiteDocumentViewSet)
router.register('device-documents', views.DeviceDocumentViewSet)
router.register('device-type-documents', views.DeviceTypeDocumentViewSet)
router.register('circuit-documents', views.CircuitDocumentViewSet)

urlpatterns = router.urls


from extras.plugins import PluginConfig

class NetboxDocuments(PluginConfig):
    name = 'netbox_documents'
    verbose_name = 'Document Storage'
    description = 'Manage site, circuit and device diagrams and documents in Netbox'
    version = '0.3.1'
    base_url = 'documents'
    default_settings = {
        "enable_site_documents": True,
        "enable_circuit_documents": True,
        "enable_device_documents": True,
        "enable_device_type_documents": True, 
        "enable_navigation_menu": True,
        "site_documents_location": "left",
        "circuit_documents_location": "left",
        "device_documents_location": "left",
        "device_type_documents_location": "left", 
    }

config = NetboxDocuments


from django import forms
from netbox.forms import NetBoxModelForm, NetBoxModelFilterSetForm
from dcim.models import Site, Device, DeviceType 
from circuits.models import Circuit
from utilities.forms import TagFilterField, CommentField, DynamicModelChoiceField
from .models import SiteDocument, DeviceDocument, DeviceTypeDocument, CircuitDocument, CircuitDocTypeChoices, SiteDocTypeChoices, DeviceDocTypeChoices, DeviceTypeDocTypeChoices 


#### Site Document Form & Filter Form
class SiteDocumentForm(NetBoxModelForm):
    comments = CommentField()

    site = DynamicModelChoiceField(
        queryset=Site.objects.all()
    )

    class Meta:
        model = SiteDocument
        fields = ('name', 'document', 'document_type', 'site', 'comments', 'tags')

class SiteDocumentFilterForm(NetBoxModelFilterSetForm):
    model = SiteDocument

    name = forms.CharField(
        required=False
    )

    site = forms.ModelMultipleChoiceField(
        queryset=Site.objects.all(),
        required=False
    )

    document_type = forms.MultipleChoiceField(
        choices=SiteDocTypeChoices,
        required=False
    )

    tag = TagFilterField(model)


#### Device Document Form & Filter Form
class DeviceDocumentForm(NetBoxModelForm):
    comments = CommentField()

    device = DynamicModelChoiceField(
        queryset=Device.objects.all()
    )

    class Meta:
        model = DeviceDocument
        fields = ('name', 'document', 'document_type', 'device', 'comments', 'tags')

class DeviceDocumentFilterForm(NetBoxModelFilterSetForm):
    model = DeviceDocument

    name = forms.CharField(
        required=False
    )

    device = forms.ModelMultipleChoiceField(
        queryset=Device.objects.all(),
        required=False
    )

    document_type = forms.MultipleChoiceField(
        choices=DeviceDocTypeChoices,
        required=False
    )

    tag = TagFilterField(model)


#### Device Type Document Form & Filter Form
class DeviceTypeDocumentForm(NetBoxModelForm):
    comments = CommentField()

    device_type = DynamicModelChoiceField(
        queryset=DeviceType.objects.all()
    )

    class Meta:
        model = DeviceTypeDocument
        fields = ('name', 'document', 'document_type', 'device_type', 'comments', 'tags')

class DeviceTypeDocumentFilterForm(NetBoxModelFilterSetForm):
    model = DeviceTypeDocument

    name = forms.CharField(
        required=False
    )

    device = forms.ModelMultipleChoiceField(
        queryset=DeviceType.objects.all(),
        required=False
    )

    document_type = forms.MultipleChoiceField(
        choices=DeviceTypeDocTypeChoices,
        required=False
    )

    tag = TagFilterField(model)


#### Circuit Document Form & Filter Form
class CircuitDocumentForm(NetBoxModelForm):
    comments = CommentField()

    circuit = DynamicModelChoiceField(
        queryset=Circuit.objects.all()
    )

    class Meta:
        model = CircuitDocument
        fields = ('name', 'document', 'document_type', 'circuit', 'comments', 'tags')

class CircuitDocumentFilterForm(NetBoxModelFilterSetForm):
    model = CircuitDocument

    name = forms.CharField(
        required=False
    )

    circuit = forms.ModelMultipleChoiceField(
        queryset=Circuit.objects.all(),
        required=False
    )

    document_type = forms.MultipleChoiceField(
        choices=CircuitDocTypeChoices,
        required=False
    )

    tag = TagFilterField(model)



from extras.plugins import PluginMenuItem
from django.conf import settings

plugin_settings = settings.PLUGINS_CONFIG.get('netbox_documents', {})

if plugin_settings.get('enable_navigation_menu'):

    menu_items = []

    # Add a menu item for Site Documents if enabled
    if plugin_settings.get('enable_site_documents'):
        menu_items.append(PluginMenuItem(
            link='plugins:netbox_documents:sitedocument_list',
            link_text='Site Documents'
        ))

    # Add a menu item for Device Documents if enabled
    if plugin_settings.get('enable_device_documents'):
        menu_items.append(
            PluginMenuItem(
                link='plugins:netbox_documents:devicedocument_list',
                link_text='Device Documents'
            )
        )

    
    # Add a menu item for Device Documents if enabled
    if plugin_settings.get('enable_device_type_documents'):
        menu_items.append(
            PluginMenuItem(
                link='plugins:netbox_documents:devicetypedocument_list',
                link_text='Device Type Documents'
            )
        )

    # Add a menu item for Circuit Documents if enabled
    if plugin_settings.get('enable_circuit_documents'):
        menu_items.append(
            PluginMenuItem(
                link='plugins:netbox_documents:circuitdocument_list',
                link_text='Circuit Documents'
            )
        )


from netbox.filtersets import NetBoxModelFilterSet
from .models import SiteDocument, DeviceDocument, DeviceTypeDocument, CircuitDocument 
from django.db.models import Q

class SiteDocumentFilterSet(NetBoxModelFilterSet):

    class Meta:
        model = SiteDocument
        fields = ('id', 'name', 'document_type', 'site')

    def search(self, queryset, name, value):
        if not value.strip():
            return queryset
        return queryset.filter(
            Q(name__icontains=value) |
            Q(document__icontains=value)
        )

class DeviceDocumentFilterSet(NetBoxModelFilterSet):

    class Meta:
        model = DeviceDocument
        fields = ('id', 'name', 'document_type', 'device')

    def search(self, queryset, name, value):
        if not value.strip():
            return queryset
        return queryset.filter(
            Q(name__icontains=value) |
            Q(document__icontains=value)
        )


class DeviceTypeDocumentFilterSet(NetBoxModelFilterSet):

    class Meta:
        model = DeviceTypeDocument
        fields = ('id', 'name', 'document_type', 'device_type')

    def search(self, queryset, name, value):
        if not value.strip():
            return queryset
        return queryset.filter(
            Q(name__icontains=value) |
            Q(document__icontains=value)
        )

class CircuitDocumentFilterSet(NetBoxModelFilterSet):

    class Meta:
        model = CircuitDocument
        fields = ('id', 'name', 'document_type', 'circuit')

    def search(self, queryset, name, value):
        if not value.strip():
            return queryset
        return queryset.filter(
            Q(name__icontains=value) |
            Q(document__icontains=value)
        )



from netbox.views import generic
from . import forms, models, tables, filtersets


### SiteDocument
class SiteDocumentView(generic.ObjectView):
    queryset = models.SiteDocument.objects.all()

class SiteDocumentListView(generic.ObjectListView):
    queryset = models.SiteDocument.objects.all()
    table = tables.SiteDocumentTable
    filterset = filtersets.SiteDocumentFilterSet
    filterset_form = forms.SiteDocumentFilterForm

class SiteDocumentEditView(generic.ObjectEditView):
    queryset = models.SiteDocument.objects.all()
    form = forms.SiteDocumentForm

class SiteDocumentDeleteView(generic.ObjectDeleteView):
    queryset = models.SiteDocument.objects.all()

### DeviceDocument
class DeviceDocumentView(generic.ObjectView):
    queryset = models.DeviceDocument.objects.all()

class DeviceDocumentListView(generic.ObjectListView):
    queryset = models.DeviceDocument.objects.all()
    table = tables.DeviceDocumentTable
    filterset = filtersets.DeviceDocumentFilterSet
    filterset_form = forms.DeviceDocumentFilterForm

class DeviceDocumentEditView(generic.ObjectEditView):
    queryset = models.DeviceDocument.objects.all()
    form = forms.DeviceDocumentForm

class DeviceDocumentDeleteView(generic.ObjectDeleteView):
    queryset = models.DeviceDocument.objects.all()


### DeviceTypeDocument
class DeviceTypeDocumentView(generic.ObjectView):
    queryset = models.DeviceTypeDocument.objects.all()

class DeviceTypeDocumentListView(generic.ObjectListView):
    queryset = models.DeviceTypeDocument.objects.all()
    table = tables.DeviceTypeDocumentTable
    filterset = filtersets.DeviceTypeDocumentFilterSet
    filterset_form = forms.DeviceTypeDocumentFilterForm

class DeviceTypeDocumentEditView(generic.ObjectEditView):
    queryset = models.DeviceTypeDocument.objects.all()
    form = forms.DeviceTypeDocumentForm

class DeviceTypeDocumentDeleteView(generic.ObjectDeleteView):
    queryset = models.DeviceTypeDocument.objects.all()

### CircuitDocument
class CircuitDocumentView(generic.ObjectView):
    queryset = models.CircuitDocument.objects.all()

class CircuitDocumentListView(generic.ObjectListView):
    queryset = models.CircuitDocument.objects.all()
    table = tables.CircuitDocumentTable
    filterset = filtersets.CircuitDocumentFilterSet
    filterset_form = forms.CircuitDocumentFilterForm

class CircuitDocumentEditView(generic.ObjectEditView):
    queryset = models.CircuitDocument.objects.all()
    form = forms.CircuitDocumentForm

class CircuitDocumentDeleteView(generic.ObjectDeleteView):
    queryset = models.CircuitDocument.objects.all()


from rest_framework import serializers

from netbox.api.serializers import NetBoxModelSerializer
from ..models import SiteDocument, DeviceDocument, DeviceTypeDocument, CircuitDocument 
from dcim.api.nested_serializers import NestedSiteSerializer, NestedDeviceSerializer, NestedDeviceTypeSerializer 
from circuits.api.nested_serializers import NestedCircuitSerializer

class SiteDocumentSerializer(NetBoxModelSerializer):

    url = serializers.HyperlinkedIdentityField(
        view_name='plugins-api:netbox_documents-api:sitedocument-detail'
    )

    site = NestedSiteSerializer()

    class Meta:
        model = SiteDocument
        fields = (
            'id', 'url', 'display', 'name', 'document', 'document_type', 'filename', 'site', 'comments', 'tags', 'custom_fields', 'created',
            'last_updated',
        )

class DeviceDocumentSerializer(NetBoxModelSerializer):

    url = serializers.HyperlinkedIdentityField(
        view_name='plugins-api:netbox_documents-api:devicedocument-detail'
    )

    device = NestedDeviceSerializer()

    class Meta:
        model = DeviceDocument
        fields = (
            'id', 'url', 'display', 'name', 'document', 'document_type', 'filename', 'device', 'comments', 'tags', 'custom_fields', 'created',
            'last_updated',
        )


class DeviceTypeDocumentSerializer(NetBoxModelSerializer):

    url = serializers.HyperlinkedIdentityField(
        view_name='plugins-api:netbox_documents-api:devicetypedocument-detail'
    )

    device_type = NestedDeviceTypeSerializer()

    class Meta:
        model = DeviceTypeDocument
        fields = (
            'id', 'url', 'display', 'name', 'document', 'document_type', 'filename', 'device_type', 'comments', 'tags', 'custom_fields', 'created',
            'last_updated',
        )

class CircuitDocumentSerializer(NetBoxModelSerializer):

    url = serializers.HyperlinkedIdentityField(
        view_name='plugins-api:netbox_documents-api:circuitdocument-detail'
    )

    circuit = NestedCircuitSerializer()

    class Meta:
        model = CircuitDocument
        fields = (
            'id', 'url', 'display', 'name', 'document', 'document_type', 'filename', 'circuit', 'comments', 'tags', 'custom_fields', 'created',
            'last_updated',
        )


from setuptools import find_packages, setup

from os import path
top_level_directory = path.abspath(path.dirname(__file__))
with open(path.join(top_level_directory, 'README.md'), encoding='utf-8') as file:
    long_description = file.read()

setup(
    name='netbox-documents',
    version='0.4.4',
    description='Manage site, circuit and device diagrams and documents in Netbox',
    long_description=long_description,
    long_description_content_type='text/markdown',
    author='Jason Yates',
    author_email='me@jasonyates.co.uk',
    url='https://github.com/jasonyates/netbox-documents',
    install_requires=[],
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
    keywords=['netbox', 'netbox-plugin', 'plugin'],
)



from netbox.api.viewsets import NetBoxModelViewSet

from .. import models
from .serializers import SiteDocumentSerializer, DeviceDocumentSerializer, DeviceTypeDocumentSerializer, CircuitDocumentSerializer 

class SiteDocumentViewSet(NetBoxModelViewSet):
    queryset = models.SiteDocument.objects.prefetch_related('tags')
    serializer_class = SiteDocumentSerializer

class DeviceDocumentViewSet(NetBoxModelViewSet):
    queryset = models.DeviceDocument.objects.prefetch_related('tags')
    serializer_class = DeviceDocumentSerializer


class DeviceTypeDocumentViewSet(NetBoxModelViewSet):
    queryset = models.DeviceTypeDocument.objects.prefetch_related('tags')
    serializer_class = DeviceTypeDocumentSerializer

class CircuitDocumentViewSet(NetBoxModelViewSet):
    queryset = models.CircuitDocument.objects.prefetch_related('tags')
    serializer_class = CircuitDocumentSerializer


from extras.plugins import PluginTemplateExtension
from django.conf import settings
from .models import SiteDocument, DeviceDocument, DeviceTypeDocument, CircuitDocument 

plugin_settings = settings.PLUGINS_CONFIG.get('netbox_documents', {})

class SiteDocumentList(PluginTemplateExtension):
    model = 'dcim.site'

    def left_page(self):

        if plugin_settings.get('enable_site_documents') and plugin_settings.get('site_documents_location') == 'left':

            return self.render('netbox_documents/sitedocument_include.html', extra_context={
                'site_documents': SiteDocument.objects.filter(site=self.context['object']),
            })

        else:
            return ""

    def right_page(self):

        if plugin_settings.get('enable_site_documents') and plugin_settings.get('site_documents_location') == 'right':

            return self.render('netbox_documents/sitedocument_include.html', extra_context={
                'site_documents': SiteDocument.objects.filter(site=self.context['object']),
            })

        else:
            return ""


class DeviceDocumentList(PluginTemplateExtension):
    model = 'dcim.device'

    def left_page(self):

        if plugin_settings.get('enable_device_documents') and plugin_settings.get('device_documents_location') == 'left':

            return self.render('netbox_documents/devicedocument_include.html', extra_context={
                'device_documents': DeviceDocument.objects.filter(device=self.context['object']),
            })

        else:
            return ""

    def right_page(self):

        if plugin_settings.get('enable_device_documents') and plugin_settings.get('device_documents_location') == 'right':

            return self.render('netbox_documents/devicedocument_include.html', extra_context={
                'device_documents': DeviceDocument.objects.filter(device=self.context['object']),
            })

        else:
            return ""


class DeviceTypeDocumentList(PluginTemplateExtension):
    model = 'dcim.devicetype'

    def left_page(self):

        if plugin_settings.get('enable_device_type_documents') and plugin_settings.get('device_type_documents_location') == 'left':

            return self.render('netbox_documents/devicetypedocument_include.html', extra_context={
                'device_type_documents': DeviceTypeDocument.objects.filter(device_type=self.context['object']),
            })

        else:
            return ""

    def right_page(self):

        if plugin_settings.get('enable_device_type_documents') and plugin_settings.get('device_type_documents_location') == 'right':

            return self.render('netbox_documents/devicetypedocument_include.html', extra_context={
                'device_type_documents': DeviceTypeDocument.objects.filter(device_type=self.context['object']),
            })

        else:
            return ""

class CircuitDocumentList(PluginTemplateExtension):
    model = 'circuits.circuit'

    def left_page(self):

        if plugin_settings.get('enable_circuit_documents') and plugin_settings.get('circuit_documents_location') == 'left':

            return self.render('netbox_documents/circuitdocument_include.html', extra_context={
                'circuit_documents': CircuitDocument.objects.filter(circuit=self.context['object']),
            })

        else:
            return ""

    def right_page(self):

        if plugin_settings.get('enable_circuit_documents') and plugin_settings.get('circuit_documents_location') == 'right':

            return self.render('netbox_documents/circuitdocument_include.html', extra_context={
                'circuit_documents': CircuitDocument.objects.filter(circuit=self.context['object']),
            })

        else:
            return ""

template_extensions = [SiteDocumentList, DeviceDocumentList, DeviceTypeDocumentList, CircuitDocumentList]


import django_tables2 as tables

from netbox.tables import NetBoxTable, columns
from .models import SiteDocument, DeviceDocument, DeviceTypeDocument, CircuitDocument 

SITE_DOCUMENT_LINK = """
<a href="{% url 'plugins:netbox_documents:sitedocument' pk=record.pk %}">
    {% firstof record.name record.filename %}
</a> (<a href="{{record.document.url}}" target="_blank">View Document</a>)
"""

CIRCUIT_DOCUMENT_LINK = """
<a href="{% url 'plugins:netbox_documents:circuitdocument' pk=record.pk %}">
    {% firstof record.name record.filename %}
</a> (<a href="{{record.document.url}}" target="_blank">View Document</a>)
"""

DEVICE_DOCUMENT_LINK = """
<a href="{% url 'plugins:netbox_documents:devicedocument' pk=record.pk %}">
    {% firstof record.name record.filename %}
</a> (<a href="{{record.document.url}}" target="_blank">View Document</a>)
"""

DEVICE_TYPE_DOCUMENT_LINK = """
<a href="{% url 'plugins:netbox_documents:devicetypedocument' pk=record.pk %}">
    {% firstof record.name record.filename %}
</a> (<a href="{{record.document.url}}" target="_blank">View Document</a>)
"""

class SiteDocumentTable(NetBoxTable):
    name = tables.TemplateColumn(template_code=SITE_DOCUMENT_LINK)
    document_type = columns.ChoiceFieldColumn()
    site = tables.Column(
        linkify=True
    )

    tags = columns.TagColumn(
        url_name='plugins:netbox_documents:sitedocument_list'
    )

    class Meta(NetBoxTable.Meta):
        model = SiteDocument
        fields = ('pk', 'id', 'name', 'document_type',  'size', 'filename', 'site', 'comments', 'actions', 'created', 'last_updated', 'tags')
        default_columns = ('name', 'document_type', 'site', 'tags')

class DeviceDocumentTable(NetBoxTable):
    name = tables.TemplateColumn(template_code=DEVICE_DOCUMENT_LINK)
    document_type = columns.ChoiceFieldColumn()
    device = tables.Column(
        linkify=True
    )

    tags = columns.TagColumn(
        url_name='dcim:sitegroup_list'
    )

    class Meta(NetBoxTable.Meta):
        model = DeviceDocument
        fields = ('pk', 'id', 'name', 'document_type',  'size', 'filename', 'device', 'comments', 'actions', 'created', 'last_updated', 'tags')
        default_columns = ('name', 'document_type', 'device', 'tags')


class DeviceTypeDocumentTable(NetBoxTable):
    name = tables.TemplateColumn(template_code=DEVICE_TYPE_DOCUMENT_LINK)
    document_type = columns.ChoiceFieldColumn()
    device_type = tables.Column(
        linkify=True
    )

    tags = columns.TagColumn(
        url_name='dcim:sitegroup_list'
    )

    class Meta(NetBoxTable.Meta):
        model = DeviceTypeDocument
        fields = ('pk', 'id', 'name', 'document_type',  'size', 'filename', 'device_type', 'comments', 'actions', 'created', 'last_updated', 'tags')
        default_columns = ('name', 'document_type', 'device_type', 'tags')

class CircuitDocumentTable(NetBoxTable):
    name = tables.TemplateColumn(template_code=CIRCUIT_DOCUMENT_LINK)
    document_type = columns.ChoiceFieldColumn()
    circuit = tables.Column(
        linkify=True
    )

    tags = columns.TagColumn(
        url_name='dcim:sitegroup_list'
    )

    class Meta(NetBoxTable.Meta):
        model = CircuitDocument
        fields = ('pk', 'id', 'name', 'document_type',  'size', 'filename', 'circuit', 'comments', 'actions', 'created', 'last_updated', 'tags')
        default_columns = ('name', 'document_type', 'circuit', 'tags')


# Generated by Django 4.0.3 on 2022-04-04 19:31

import django.core.serializers.json
from django.db import migrations, models
import django.db.models.deletion
import netbox_documents.utils
import taggit.managers


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('dcim', '0153_created_datetimefield'),
        ('circuits', '0034_created_datetimefield'),
        ('extras', '0072_created_datetimefield'),
    ]

    operations = [
        migrations.CreateModel(
            name='SiteDocument',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False)),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                ('custom_field_data', models.JSONField(blank=True, default=dict, encoder=django.core.serializers.json.DjangoJSONEncoder)),
                ('name', models.CharField(blank=True, max_length=100)),
                ('document', models.FileField(upload_to=netbox_documents.utils.file_upload)),
                ('document_type', models.CharField(max_length=30)),
                ('comments', models.TextField(blank=True)),
                ('site', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='documents', to='dcim.site')),
                ('tags', taggit.managers.TaggableManager(through='extras.TaggedItem', to='extras.Tag')),
            ],
            options={
                'ordering': ('-created', 'name'),
            },
        ),
        migrations.CreateModel(
            name='DeviceDocument',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False)),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                ('custom_field_data', models.JSONField(blank=True, default=dict, encoder=django.core.serializers.json.DjangoJSONEncoder)),
                ('name', models.CharField(blank=True, max_length=100)),
                ('document', models.FileField(upload_to=netbox_documents.utils.file_upload)),
                ('document_type', models.CharField(max_length=30)),
                ('comments', models.TextField(blank=True)),
                ('device', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='documents', to='dcim.device')),
                ('tags', taggit.managers.TaggableManager(through='extras.TaggedItem', to='extras.Tag')),
            ],
            options={
                'ordering': ('name',),
            },
        ),
        migrations.CreateModel(
            name='CircuitDocument',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False)),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                ('custom_field_data', models.JSONField(blank=True, default=dict, encoder=django.core.serializers.json.DjangoJSONEncoder)),
                ('name', models.CharField(blank=True, max_length=100)),
                ('document', models.FileField(upload_to=netbox_documents.utils.file_upload)),
                ('document_type', models.CharField(max_length=30)),
                ('comments', models.TextField(blank=True)),
                ('circuit', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='documents', to='circuits.circuit')),
                ('tags', taggit.managers.TaggableManager(through='extras.TaggedItem', to='extras.Tag')),
            ],
            options={
                'ordering': ('name',),
            },
        ),
    ]
